package com.mgh.pmdm.parques.view.ui

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.core.widget.doAfterTextChanged
import androidx.fragment.app.activityViewModels
import androidx.lifecycle.ViewModelProvider
import com.google.android.material.snackbar.Snackbar
import com.mgh.pmdm.parques.R
import com.mgh.pmdm.parques.databinding.FragmentSecondBinding
import com.mgh.pmdm.parques.db.Park
import com.mgh.pmdm.parques.view.dialogs.MiDialogFragment
import com.mgh.pmdm.parques.viewmodel.MainViewModel


class SecondFragment : Fragment(), MiDialogFragment.OnOKOrCancelListener {

    // Definimos el binging
    private var _binding: FragmentSecondBinding? = null
    private val binding get() = _binding!!

    // Referencia al ViewModel
    private lateinit var viewModel: MainViewModel

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Preparamos únicamente la vista
        _binding = FragmentSecondBinding.inflate(inflater, container, false)
        populateSpinner()
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Instanciamos el ViewModel, utilizando la clase ViewModelProvider
        // En lugar de this, vamos a utilizar requireActivity(), para trabajar en el ámbito
        // de la actividad y poder compartir el ViewModel
        viewModel = ViewModelProvider(requireActivity()).get(MainViewModel::class.java)

        // Creamos un nuevo método para preparar todos los
        // observers y cómo se actualizan.
        prepareObservers()

        // Gestor del evento onClick sobre el botón de guardar
        binding.btSave.setOnClickListener {
            // Preparamos el diálogo de confirmación
            val miDialogo = MiDialogFragment(
                resources.getString(R.string.SaveData),
                resources.getString(R.string.AskSaveData),
                this
            )
            miDialogo.show(requireActivity().supportFragmentManager, "miDialogo")
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    /*********************************
     * Métodos específicos de la clase
     **********************************/

    fun prepareObservers() {
        // Definición de observadores a los LiveData para ser notificados de los cambios

        // Observer para el parque en edición:
        // Actualizamos las vistas, y al mismo tiempo, cuando se modifican
        // actualizamos el current_park
        viewModel.currentPark.observe(viewLifecycleOwner) {
            it?.let { mypark ->
                //  TextViews
                binding.Name.setText(mypark.name)
                binding.Name.doAfterTextChanged { mypark.name = it.toString() }
                binding.Description.setText(mypark.desc)
                binding.Description.doAfterTextChanged { mypark.desc = it.toString() }
                binding.Phone.setText(mypark.phone)
                binding.Phone.doAfterTextChanged { mypark.phone = it.toString() }
                binding.webSite.setText(mypark.webiste)
                binding.webSite.doAfterTextChanged { mypark.webiste = it.toString() }

                // Actualización del valor de los spinners
                for (i in 0..binding.SpinnerOpeningTime.adapter.count)
                    if (binding.SpinnerOpeningTime.adapter.getItem(i).equals(mypark.openingTime)) {
                        binding.SpinnerOpeningTime.setSelection(i)
                        break
                    }

                // Listeners para los cambios en los spinners
                binding.SpinnerOpeningTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.openingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }

                for (i in 0..binding.SpinnerClosingTime.adapter.count)
                    if (binding.SpinnerClosingTime.adapter.getItem(i).equals(it.closingTime)) {
                        binding.SpinnerClosingTime.setSelection(i)
                        break
                    }

                binding.SpinnerClosingTime.onItemSelectedListener = object :
                    AdapterView.OnItemSelectedListener {
                    override fun onItemSelected(
                        parent: AdapterView<*>?,
                        view: View?,
                        position: Int,
                        id: Long
                    ) {
                        it.closingTime = parent?.getItemAtPosition(position).toString()
                    }

                    override fun onNothingSelected(parent: AdapterView<*>?) {
                        // Nada por hacer
                    }
                }

                // Actualización de los Checkboxes
                if (mypark.sports == true) binding.cbSport.isChecked = true

                binding.cbSport.setOnCheckedChangeListener { _, isChecked ->
                    mypark.sports = isChecked
                }

                if (mypark.children == true) binding.cbChildren.isChecked = true

                binding.cbChildren.setOnCheckedChangeListener { _, isChecked ->
                    mypark.children = isChecked
                }

                if (mypark.Bar == true) binding.cbBar.isChecked = true
                binding.cbBar.setOnCheckedChangeListener { _, isChecked ->
                    mypark.Bar = isChecked
                }

                if (mypark.Pets == true) binding.cbMascotas.isChecked = true
                binding.cbMascotas.setOnCheckedChangeListener { _, isChecked ->
                    mypark.Pets = isChecked
                }

            }

            // ImageView, de momento fija
            binding.Image.setImageResource(R.drawable.appimg)
            Log.d("Second Fragment", "After Image")

        }


        // Observa cuando se ha añadido un nuevo parque para mostrar el Snackbar
        viewModel.parkSaved.observe(viewLifecycleOwner) { saved ->
            saved?.let {
                // Limpiamos el valor para no entrar en un bucle infinito
                viewModel.parkSaved.value=null
                Snackbar.make(
                    binding.root,
                    resources.getString(R.string.dataSaved),
                    Snackbar.LENGTH_LONG
                ).show()
            }
        }

        // Observa cuando se ha modificado un parque para mostrar el Snackbar
        viewModel.parkUpdated.observe(viewLifecycleOwner) { saved ->
            saved?.let {
                // Limpiamos el valor para no entrar en un bucle infinito
                viewModel.parkUpdated.value=null
                Snackbar.make(
                    binding.root,
                    resources.getString(R.string.updatedSaved),
                    Snackbar.LENGTH_LONG
                ).show()
            }
        }
    }

    // Método específico para poblar el spinner para la hora de cierre
    private fun populateSpinner() {
        ArrayAdapter.createFromResource(
            requireActivity(),
            R.array.horas,
            android.R.layout.simple_spinner_dropdown_item
        ).also { adapter ->
            adapter.setDropDownViewResource(android.R.layout.simple_spinner_item)
            binding.SpinnerClosingTime.adapter = adapter
        }
    }

    override fun onPositiveClick() {
        // Creamos un nuevo parque con la información del formulario
        val newPark = Park(
            name = binding.Name.text.toString(),
            desc = binding.Description.text.toString(),
            phone = binding.Phone.text.toString(),
            webiste = binding.webSite.text.toString(),
            openingTime = binding.SpinnerOpeningTime.selectedItem.toString(),
            closingTime = binding.SpinnerClosingTime.selectedItem.toString(),
            sports = binding.cbSport.isChecked,
            children = binding.cbChildren.isChecked,
            Bar = binding.cbBar.isChecked,
            Pets = binding.cbMascotas.isChecked
        )

        viewModel.savePark(newPark)

    }


    override fun onCancelClick() {
        // Si pulsa cancelar mostramos un toast
        Toast.makeText(
            requireActivity().applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }


}