package com.mgh.pmdm.parques.viewmodel

import android.app.Application
import android.util.Log
import android.view.View
import androidx.lifecycle.*
import com.mgh.pmdm.parques.db.Park
import com.mgh.pmdm.parques.repository.ParkRepository
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class MainViewModel(application: Application) : AndroidViewModel(application) {

    // Referencia al repositorio (para simplificar el código posterior)
    val repository = ParkRepository.getInstance(application.applicationContext)

    // Referencia a la propia clase, para poder referenciarla
    // dentro de los callbacks (ya que ocultan "this")
    private val me = this

    /*************************************************/
    /* Campos LiveData observables desde la interfaz */
    /*************************************************/

    // Parque que se está editando actualmente
    // (atributo de respaldo y de acceso públic)
    private var _currentPark = MutableLiveData<Park?>()
    val currentPark: LiveData<Park?> = _currentPark


    // Ponte a Prueba 3
    // Lista de parques, con todos los parques
    val parkListAll: LiveData<List<Park>> by lazy {
        repository.getParks()
    }

    // Ponte a Prueba 3
    // Lista de parques filtrada
    val parkListFiltered: LiveData<List<Park>> by lazy {
        repository.getParksForChildren()
    }

    // Ponte a Prueba 3
    // Referencia a la lista de parques actual
    var parkList = parkListAll

    // Indicadores para advertir a la interfaz de que el parque se
    // guardó o actualizó. Ojo con el espacio antes de la asignación!
    var parkSaved: MutableLiveData<Boolean> = MutableLiveData()
    var parkUpdated: MutableLiveData<Boolean> = MutableLiveData()

    // Livedata para notificar la eliminación de un elemento a la interfaz
    var deletedPos: MutableLiveData<Int> = MutableLiveData()

    // Livedata para gestionar los clics
    val parkLongClicked: MutableLiveData<Park> by lazy {
        MutableLiveData<Park>()
    }
    val parkClicked: MutableLiveData<Park> by lazy {
        MutableLiveData<Park>()
    }


    // LiveData para el adaptador mediante un atributo de soporte privado:
    // El adaptador del RecyclerView también será un LiveData para
    // poder ser observable desde la vista.
    private val _adaptador = MutableLiveData<AdaptadorParques>().apply {
        // Mediante value (setValue) establecemos el valor que contendrá el MutableLiveData
        // de tipo AdaptadorParques. Aqui proporcionamos también los callbacks que
        // se deben invocar cuando se produzcan clicks.
        value = AdaptadorParques(
            me, // Referencia al  propio ViewModel
            { park: Park, v: View -> parkClickedManager(park, v) },
            { park: Park, v: View -> ParkLongClickedManager(park, v) }
        )
    }

    // Definimos la propiedad para acceder al adaptador desde la vista
    // (_adapter es un atributo de soporte provado)
    val adaptador: MutableLiveData<AdaptadorParques> = _adaptador


    /********************************/
    /* Declaración de Métodos       */
    /********************************/

    fun cleanPark() {
        // Resetea el valor del parque actual en edición
        _currentPark.value = null
    }

    fun setCurrentPark(park: Park) {
        // Establece el valor del parque actual en edición
        _currentPark.value = park.copy()
    }

    // Callbacks para la gestión de eventos. Recordad que aunque
    // el evento lo gestiona el FirstFragment, debemos notificar
    // cuando se producen los clicks sobre cada elemento.

    private fun parkClickedManager(park: Park, v: View) {
        // Para establecer el valor del liveData utilizamos value
        parkClicked.value = park
    }

    private fun ParkLongClickedManager(park: Park, v: View): Boolean {
        // Para establecer el valor del liveData utilizamos Value
        parkLongClicked.value = park
        return true
    }

    /* Métodos para CRUD */

    fun removePark(park: Park) {
        // Elimina un parque de la lista
        // Se modificará el modelo a través del repositorio, y
        // se notificará al Adaptador para que se refresque.

        viewModelScope.launch(Dispatchers.IO) {
            repository.removePark(park)

            // Notificamos al adaptador la eliminación
            parkList.value?.indexOf(park)?.let {
                deletedPos.postValue(it);
            }
        }

    }

    fun savePark(park: Park) {
        // Método para guardar el parque

        viewModelScope.launch(Dispatchers.IO) {
            // Si existe, deberemos modificarlo
            if (_currentPark.value != null) {
                repository.update(_currentPark.value as Park)

                // Indicamos al adaptador que se ha modificado el elemento
                parkList.value?.indexOf(_currentPark.value)
                    ?.let { adaptador.value?.notifyItemChanged(it) }

                // Activamos el indicador parkUpdate para notificar a la interfaz
                parkUpdated.postValue(true)
            } else { // Si no existe, deberemos añadirlo
                // Guardamos el parque a través del repositorio
                repository.add(park)
                // Actualizamos el CurrentPark con una copia del parque nuevo

                _currentPark.postValue(park.copy())

                // Notificamos al adaptador que se ha modificado el dataset
                adaptador.value?.notifyDataSetChanged()
                //parkList.value?.size
                //    ?.let { adaptador.value?.notifyItemInserted(it + 1) }

                // Activamos el indicador parkSaved para notificar a la
                // interfaz que se ha añadido un elemento
                parkSaved.postValue(true)
            }


        }

    }


    // Ponte a Prueba 3
    // Método para activar o desactivar el filtro
    fun changeParkList(checked:Boolean) {
        if (checked)
            parkList = parkListFiltered
        else
            parkList = parkListAll

        adaptador.value?.notifyDataSetChanged()

    }

}